/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.maven;

import org.apache.commons.lang.StringUtils;
import org.apache.maven.project.MavenProject;

import java.io.File;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.List;
import java.util.Locale;

public abstract class AbstractCollector implements Collector{
  protected double parseNumber(String number, Locale locale) throws ParseException {
    if (number.equals("")){
      return Double.NaN;
    }
    return NumberFormat.getNumberInstance(locale).parse(number).doubleValue();
  }

  protected double parseNumber(String number) throws ParseException {
    return parseNumber(number, Locale.getDefault());
  }

  protected double scaleValue(double value) {
    return scaleValue(value, 2);
  }

  protected double scaleValue(double value, int decimals) {
    BigDecimal bd = new BigDecimal(value);
    return bd.setScale(decimals, RoundingMode.HALF_UP).doubleValue();
  }

  protected XmlReportParser loadAndParseXmlReport(File xmlFile) throws MissingDataException {
    if (!xmlFile.exists()) {
      throw new MissingDataException("File does not exist : " + xmlFile.toString());
    }
    XmlReportParser parser = new XmlReportParser();
    parser.parse(xmlFile);
    return parser;
  }

  protected File findFileFromBuildDirectory( MavenProject project, String filename ) {
    String xmlOutputDirectory = project.getBuild().getDirectory() + "/" + filename;
    java.io.File file = new java.io.File( xmlOutputDirectory );
    if ( !file.exists() ) {
      file = null;
    }
    return file;
  }

  protected String[] getPackageAndClass(String absoluteFilename, List<String> sourceDirs) {
    String unixFilename = absoluteFilename.replace('\\', '/');
    String filename = StringUtils.substringAfterLast(unixFilename, "/");
    String namespace = StringUtils.substringBeforeLast(unixFilename, filename);

    if (! filename.toLowerCase(Locale.ENGLISH).endsWith(".java")) {
      return null;
    }

    // namespace == "/home/project/org/sonar/maven/" or "/home/project/" (no package), filename="MyClass.java"
    for (String sourceDir : sourceDirs) {
      sourceDir = sourceDir.replace('\\', '/');
      if (!sourceDir.endsWith("/")) {
        sourceDir += "/";
      }
      if (namespace.startsWith(sourceDir) || namespace.contains(sourceDir)) {
        namespace = StringUtils.substringAfter(namespace, sourceDir);
        // namespace == "/org/sonar/maven/" or "org/sonar/maven/" or "/" or "" (no package)

        if (namespace.startsWith("/")) {
          namespace = StringUtils.substringAfter(namespace, "/");
        }
        if (namespace.endsWith("/")) {
          namespace = StringUtils.substringBeforeLast(namespace, "/");
        }
        // namespace == "org/sonar/maven" or ""

        namespace = namespace.replace('/', '.');
        // namespace == 'org.sonar.maven' or ''

        if ("".equals(namespace)) {
          namespace = null;
        }

        return new String[]{namespace, StringUtils.substringBeforeLast(filename, ".")};
      }
    }

    return null;
  }
}